;-------------------------------------------------------------------------------
;	FILE:			StopStart-ISR.asm
;
;	Description:	ISR for Stop/Start w/Boost 3PhIL DC/DC
;
;	Version: 		0.1
;
;   Target:  		TMS320F2803x(PiccoloB)
;
;-------------------------------------------------------------------------------
;   Copyright Texas Instruments  2014
;-------------------------------------------------------------------------------
;   Revision History:
;-------------------------------------------------------------------------------
; 	Date		| Author	| Description
;-------------------------------------------------------------------------------
; 	2014 Dec 12	| HN       	| Created new file
;-------------------------------------------------------------------------------

	;Gives peripheral addresses visibility in assembly
    .cdecls   C,LIST,"PeripheralHeaderIncludes.h"

	;include C header file - sets INCR_BUILD (used in conditional builds)
	.cdecls C,NOLIST, "StopStart-Settings.h"

	; MACRO includes
	.include "ADCDRV_1ch.asm"
	.include "ADCDRV_8ch.asm"
	.include "CNTL_2P2Z.asm"
	.include "Boost3PhIL_PwmDrv.asm"

;**********************************************************************************
; Declare Public functions for External Reference
;**********************************************************************************
				.def 	_ISR_Init
				.def 	_ISR_Run

;**********************************************************************************
; Variable declaration
;**********************************************************************************	
; All Terminal modules initially point to the ZeroNet to ensure a known
; start state. Pad extra locations to accomodate unwanted ADC results.
ZeroNet			.usect "Net_terminals",8,1,1
DummyNet	 	.usect "Net_terminals",7,1,1
IsrClkCtr		.usect "Net_terminals",1

				.text
;---------------------------------------------------------
; ISR Initialisation
;---------------------------------------------------------
_ISR_Init:

; Clear the ZeroNet
	MOVL 	XAR2,#ZeroNet
	RPT		#7 ; 8 times
	||MOV	*XAR2++, #0

	;---------------------------------------------------------
	.if(INCR_BUILD = 1) ; Open-Loop
		ADCDRV_8ch_INIT			1, 2, 3, 5, 6, 7, 9, 10
		ADCDRV_1ch_INIT 		11
		Boost3PhIL_PwmDrv_INIT 	1		; EPWM1, EPWM2, EPWM3
	.endif
	;---------------------------------------------------------
	.if(INCR_BUILD = 2) ; Closed-Loop
		ADCDRV_8ch_INIT			1, 2, 3, 5, 6, 7, 9, 10
		ADCDRV_1ch_INIT 		11
		CNTL_2P2Z_INIT			1
		Boost3PhIL_PwmDrv_INIT 	1		; EPWM1, EPWM2, EPWM3
	.endif
	;---------------------------------------------------------

	LRETR

			.sect "ramfuncs"

;---------------------------------------------------------
; ISR Run
;---------------------------------------------------------
_ISR_Run:	;(13 cycles to get to here from ISR trigger)
	;CONTEXT_SAVE
	PUSH AR1H:AR0H ; 32-bit
	PUSH XAR2 ; 32-bit
	PUSH XAR3 ; 32-bit
	PUSH XAR4 ; 32-bit
;-- Comment these to save cycles --------
	PUSH XAR5 ; 32-bit
	PUSH XAR6 ; 32-bit
	PUSH XAR7 ; 32-bit
;----------------------------------------
	PUSH XT	  ; 32-bit
	NOP		

	;---------------------------------------------------------
	.if(INCR_BUILD = 1) ; Open-Loop
		ADCDRV_8ch 	1, 2, 3, 5, 6, 7, 9, 10
		ADCDRV_1ch  11
		Boost3PhIL_PwmDrv 1				; EPWM1, EPWM2, EPWM3
	.endif
	;---------------------------------------------------------
	.if(INCR_BUILD = 2) ; Closed-Loop
		ADCDRV_8ch 	1, 2, 3, 5, 6, 7, 9, 10
		ADCDRV_1ch  11

		.ref	_No_2p2z
		MOVW	DP, #(_No_2p2z)
		MOV		AL, @(_No_2p2z)
		CMPB 	AL,#0x1
		B		SKIP_2P2Z, EQ			; If equal - coefficients are being changed in the slower loop --> don't execute 2P2Z

		CNTL_2P2Z		1				; Voltage Controller
SKIP_2P2Z:
		Boost3PhIL_PwmDrv 1				; EPWM1, EPWM2, EPWM3
	.endif
	;---------------------------------------------------------

;===================================
EXIT_ISR
;===================================
; Interrupt management before exit
	MOVW 	DP,#_EPwm1Regs.ETCLR
	MOV 	@_EPwm1Regs.ETCLR,#0x01		; Clear EPWM1 Int flag
	MOVW 	DP,#_PieCtrlRegs.PIEACK		; Acknowledge PIE interrupt Group 3
	MOV 	@_PieCtrlRegs.PIEACK, #0x4

; Restore context & return
	POP XT
;-- Comment these to save cycles ---
	POP XAR7
	POP XAR6
	POP XAR5
;-----------------------------------
	POP XAR4
	POP XAR3
	POP XAR2
	POP AR1H:AR0H

	IRET						

