;===============================================================================
;	Boost3PhIL_PwmDrv n
; 			driver block for Boost DC/DC with 3 phase interleaved with Adjust
;===============================================================================
;	FILE:			Boost3PhIL_PwmDrv.asm
;
;	Description:	Driver for a 3 phase interleaved Boost with Adjust
;
;	Version: 		0.1
;
;   Target:  		TMS320F2803x
;-------------------------------------------------------------------------------
;  	Copyright Texas Instruments  2014
;-------------------------------------------------------------------------------
; 	Revision History:
;-------------------------------------------------------------------------------
; 	Date		| Author	| Description
;-------------------------------------------------------------------------------
; 	2014 Oct 08	| HN       	| Created current macro block.
;-------------------------------------------------------------------------------
;		 	 	 _______________________
;				|Boost3PhIL_PwmDrv n   |
;				|~~~~~~~~~~~~~~~~~~~~~~|
;		 	 -->| Duty	      ePWM(n)A |--> (Boost Phase 1 switch, 0 deg)
;			 	| 		  	     	   |
;			 -->| Adj	    ePWM(n+1)A |--> (Boost Phase 2 switch, 120 deg)
;			 	| 		  	     	   |
;			    |     	    ePWM(n+2)A |--> (Boost Phase 3 switch, 240 deg)
;				|______________________|
;
;	Duty, Adj, are pointers to signal nets.
;   Peripheral Resources used: ePWM(n)A, ePWM(n+1)A, ePWM(n+2)A, ePWM(n+3)A channels
;
;===============================================================================

;====================================
Boost3PhIL_PwmDrv_INIT	.macro	n
;====================================

; Variable Declarations
;-------------------------------------------------------------------------------
_Boost3PhIL_Duty:n:A		.usect "Boost3PhIL_data",2,1,1	; Input Terminal 1
_Boost3PhIL_Duty:n:B		.usect "Boost3PhIL_data",2,1,1 	; Input Terminal 2
_Boost3PhIL_Duty:n:C		.usect "Boost3PhIL_data",2,1,1 	; Input Terminal 3
_Boost3PhIL_period:n:		.usect "Boost3PhIL_data",2,1,1 	; full period count value

;Publish Terminal Pointers for access from the C environment
;-------------------------------------------------------------------------------
					.def 	_Boost3PhIL_Duty:n:A
					.def 	_Boost3PhIL_Duty:n:B
					.def	_Boost3PhIL_Duty:n:C
					.def 	_Boost3PhIL_period:n:

	; set input terminals to point to ZeroNet
		MOVL	XAR2, #ZeroNet
		MOVW	DP, #_Boost3PhIL_Duty:n:A
		MOVL	@_Boost3PhIL_Duty:n:A, XAR2
		MOVL	@_Boost3PhIL_Duty:n:B, XAR2
		MOVL	@_Boost3PhIL_Duty:n:C, XAR2

		MOVW 	DP,#_EPwm:n:Regs.TBPRD
		MOVL	ACC, @_EPwm:n:Regs.TBPRD
		MOV		AL,#0
		MOVW	DP,#_Boost3PhIL_period:n:
		MOVL	@_Boost3PhIL_period:n:, ACC
		.endm

;================================
Boost3PhIL_PwmDrv	.macro	n
;================================
	
	; Set up address pointers:
		MOVW	DP, #_Boost3PhIL_Duty:n:A
		MOVL 	XAR3, @_Boost3PhIL_Duty:n:A		; Net pointer to Duty (XAR0)
		MOVL 	XAR4, @_Boost3PhIL_Duty:n:B		; Net pointer to Adj (XAR1)
		MOVL 	XAR5, @_Boost3PhIL_Duty:n:C		; Net pointer to Adj (XAR1)
      	MOVL    XT,@_Boost3PhIL_period:n:

		QMPYL 	ACC,XT,*XAR3					; ACC= (I8Q24) * (I16Q16) = (I24Q40): upper 32-bits -> ACC = (I24Q8)
 		SFR		ACC,#8						    ; ACC>>8: AL = duty,

	; Update CMP value for ePWM(n)A [Phase 1]
		MOVW	DP, #_EPwm:n:Regs.CMPA
		MOVL	XAR1, #_EPwm:n:Regs.CMPA.half.CMPA
	    MOV     *XAR1,AL
		ADDB 	XAR1, #0x1						; point to CMPB
		SFR		ACC, #1							; Divide by 2 to calulate CMPB for SOC
	    MOV     *XAR1,AL
		
        QMPYL 	ACC,XT,*XAR4   					; ACC= (I8Q24) * (I16Q16) = (I24Q40): upper 32-bits -> ACC = (I24Q8)
	    SFR		ACC,#8						    ; ACC>>8: AL = duty,
	; Update CMP value for ePWM(n+1)A [Phase 2]
		ADDB 	XAR1, #0x3F						; Pointer to ePWM(n+1)-CMPA compare reg (XAR1)
	    MOV     *XAR1,AL
		ADDB 	XAR1, #0x1						; point to CMPB
		SFR		ACC, #1							; Divide by 2 to calulate CMPB for SOC
	    MOV     *XAR1,AL

        QMPYL 	ACC,XT,*XAR5   					; ACC= (I8Q24) * (I16Q16) = (I24Q40): upper 32-bits -> ACC = (I24Q8)
	    SFR		ACC,#8						    ; ACC>>8: AL = duty,
	; Update CMP value for ePWM(n+1)A [Phase 2]
		ADDB 	XAR1, #0x3F						; Pointer to ePWM(n+1)-CMPA compare reg (XAR1)
	    MOV     *XAR1,AL
	    ADDB 	XAR1, #0x1						; point to CMPB
		SFR		ACC, #1							; Divide by 2 to calulate CMPB for SOC
	    MOV     *XAR1,AL

		.endm


